<?php
// Parent Für Modules und Templates
abstract class packageControllerTable {
    public static $attributes;
    public static $table;
   
    protected $id;
    protected $keyValues;

    // Konstructor
  
    public function __construct($id=null) {
        if(!$id) return;
        $this->id = $id;
        $this->fetchValues();
    }

    // public functions

    // setzt wert in db sowie im object / gibt true bei success oder false bei fehlschlag zurück

    public function setValue($key="", $value="") {
        if($key == "id" || !$key || !in_array($key, static::$attributes)) return false;
        
        // setze object wert
        $this->keyValues[$key] = $value;

        // setzt datenbank
        $sql = rex_sql::factory();
        $query = "UPDATE " . static::$table . " SET " . $key . " = :value WHERE id = :id";
        $sql->prepareQuery($query);

        $sql->execute([
            ":value" => $value,
            ":id" => $this->id
        ]);

        return true;
    }

    // gibt einen Wert zu dem key zurück / false bei fehlschlag

    public function getValue($key="") {
        if(!$key) return false;
        if($key == "id") return $this->id;
        if(!in_array($key, static::$attributes)) return false;
        return $this->keyValues[$key];
    }

    // status des datenbank eintrags wird auf 0 gesetzt

    public function delete() {
        $sql = rex_sql::factory();
        $query = "UPDATE " . static::$table . " SET status = 0 WHERE id = :id";
        $sql->prepareQuery($query);
        
        $sql->execute([
            ":id" => $this->id
        ]);
    }

    // private functions

    // Setze alle Werte aus der Datenbank

    private function fetchValues() {
        $sql = rex_sql::factory();
        $query = "SELECT * FROM " . static::$table . " WHERE id = :id LIMIT 1";
        
        $sql->prepareQuery($query);
        $sql->execute([
            ":id" => $this->id
        ]);

        if(!$sql->getRow()) throw new Exception(static::$table . " does not contain an element with an id of " . $this->id);

        $this->keyValues = [];

        foreach(static::$attributes as $attribute) {
            if(!$sql->getValue($attribute)) continue;
            $this->keyValues[$attribute] = $sql->getValue($attribute);
        }
    }

    // static functions

    // neuer eintrag in der datenbank gibt die neue id zurück / bei fehler ist rückgabe 0

 

    public static function add($data=[]) {
        $sql = rex_sql::factory();

        // überprufe daten auf richtigkeit
        foreach(static::$attributes as $attribute) {
            if(!isset($data[$attribute])) return false;
        }

        // behandle arrays indem sie mit json_encode encoded werden
        foreach($data as $key => $value) {
            if(is_array($value)) $data[$key] = json_encode($value);
        }
        
        $length = count(static::$attributes);

        $keys = "(";
        $values = "(";
        $queryData = [];

        for($i = 0; $i < $length; $i++) {
            $attribute = static::$attributes[$i];

             // keys
            $keys .= $attribute;
            $keys .= ($i + 1 < $length) ? "," : ")";

            // values
            $values .= ":" . $attribute;
            $values .= ($i + 1 < $length) ? "," : ")";

            // data
            $queryData[":" . $attribute] = $data[$attribute];
        }
       
        $query = "INSERT INTO " . static::$table . " " . $keys . " VALUES " . $values;
        $sql->prepareQuery($query);
        $sql->execute($queryData);

        $query = "SELECT id from " . static::$table;
        $sql->setQuery($query);
        return $sql->getRows();
    }

    // gibt alle active einträge zurück

    public static function getList() {
        $sql = rex_sql::factory();
        $query = "SELECT * FROM " . static::$table . " WHERE status = 1";
        $sql->setQuery($query);
        
        $result = [];

        // loop über alle activen einträge
        for($i = 0; $i < $sql->getRows(); $i++) {
            $entry = [];
            $entry["id"] = $sql->getValue("id");

            // loop über alle attribute
            foreach(static::$attributes as $attribute) {

                if(!$sql->getValue($attribute)) {
                    $entry[$attribute] = "";
                } else {
                    $entry[$attribute] = $sql->getValue($attribute);
                }
            }
            array_push($result, $entry);
            $sql->next();
        }

        return $result;
    }

}