<?php

// überprüfe ob alle Anforderungen an Module/Templates erfüllt sind 
// damit sie nachher installiert werden können
// sind keine Anforderungen angegeben oder ausgelassen werden diese automatisch als unwichtig angeshen
function checkRequirements($data = "") {
    if(!$data) return;
    $data = json_decode($data);
    
    // php handling
    if($data->php) {
        if( !(version_compare(PHP_VERSION, $data->php) >= 0) ) {
            throw new Exception("php version should be '". $data->php ."' but is '". PHP_VERSION ."'");
        }
    }
    
    // redaxo handling
    if($data->redaxo) {
        $REDAXO_VERSION = rex_config::get("core", "version");

        if( !(version_compare($REDAXO_VERSION, $data->redaxo) >= 0) ) {
            throw new Exception("redaxo version should be '". $data->redaxo ."' but is '". $REDAXO_VERSION ."'");
        }
    }
   
    // addons handling
    if($data->addons) {
        $addons = (array) $data->addons;

        $messages = [];

        foreach($addons as $name => $version) {

            // does addon exists?
            if (rex_addon::exists($name)) {
                $addon = rex_addon::get($name);
                
                // is addon installed?
                if(!$addon->isInstalled()) {
                    array_push($messages, "addon '" .$name . "' is not installed");
                    continue;
                }

                // is activated?
                if(!$addon->isAvailable()) { 
                    array_push($messages, "addon '" .$name . "' is not activated");
                    continue;
                }

                // does addon has the right version
                if( !(version_compare($addon->getProperty("version"), $version) >= 0) ) {
                    array_push($messages, "addon '" .$name . "' version shoulde be '".$version."' but is '".$addon->getProperty("version")."'");
                }

            } else {
                array_push($messages, "addon '" .$name . "' is not downloaded");
            }
        } 

        if(count($messages)) {
            $e = implode(", ", $messages);
            throw new Exception($e);
        }
    }
}