<?php

// validierung des TMP Folders
function validateTmpFolder($tmp_folder = "", $root_folder = "") {
    if(!$tmp_folder || !$root_folder) throw new Exception("validateTmpFolder needs parameters");

    // bekomme metadata.json des root folders
    $data = getMetaData($tmp_folder);
    if(!$data) throw new Exception("no root metadata");

    // neuer eintrag in der datenbank mit den daten der metadata.json
    if(!$package_id = packageControllerPackage::add($data)) throw new Exception("data import from metadata.json failed");

    // validiere modules und templates folders
    validateSubFolder("module", $package_id, $tmp_folder . "modules/", $root_folder . "modules/");
    validateSubFolder("template", $package_id, $tmp_folder . "templates/", $root_folder . "templates/");

    // lösche den tmp folder
    rex_dir::delete($tmp_folder);
} 

// prüfe type "module" oder "template", Invalide Ordner werden ignoriert
function validateSubFolder($type = "", $package_id = null, $package_folder = "", $destiny_folder = "") {
    if($type != "template" && $type != "module") return;
    if(!$package_id) return;
    if(!$package_folder || !is_dir($package_folder)) return;
    if(!$destiny_folder || !is_dir($destiny_folder)) return;
    
     // remove . & ..
    $folderNames = array_slice(scandir($package_folder), 2);

    // gehe über folder zb. templates/template1/
    foreach($folderNames as $subfolder) {

        // prüfe ob zb. bei type module die output.txt und input.txt existieren
        if(!hasRequiredFiles($type, $package_folder . $subfolder . "/")) continue;

        // bekomme metadaten des folders
        $data = getMetaData($package_folder . $subfolder . "/");

        if(!$data) echo "no modules metadata in " . $subfolder;
        $data["package_id"] = $package_id;

        // neue datenbank einträge für den jeweiligen type
        if($type == "module") {
            if(!$entry_id = packageControllerModule::add($data)) throw new Exception("data import from " . $subfolder . " metadata.json failed");
            
        } else if($type == "template") {
            if(!$entry_id = packageControllerTemplate::add($data)) throw new Exception("data import from " . $subfolder . " metadata.json failed");
        
        } 

        // datenbank eintrag sollte schon gesetzt sein
        if(!isset($entry_id) || !$entry_id) continue;

        // Kopiere aus tmp folder in den richtigen ordner mit namen [id] name
        $name = "[" . $entry_id . "] " . $data["name"];
    
        rex_dir::copy($package_folder . $subfolder, $destiny_folder . $name);
        
    }
}


// helper functions ///////////////////////////////////////////////////////

// überprüfe ob bei type module output.txt und inpu.txt vorhanden sind; bei type template muss nur die template.txt vorhanden sein
function hasRequiredFiles($type = "", $folder = "") {
    if(!$type) return false;
    if(!$folder || !is_dir($folder)) return false;

    $files = array_slice(scandir($folder), 2);

    if($type == "module") {

        if(in_array("output.txt", $files) && in_array("input.txt", $files)) {
            return true;
        } else {
            return false;
        }

    } else if($type == "template") {

        if(in_array("template.txt", $files)) {
            return true;
        } else {
            return false;
        }

    } 

    return false;
}

// bekomme meta data von folder als array / ansosnten false
function getMetaData($folder = "") {
    if(!$folder || !is_dir($folder)) return false;
    $file = $folder . "metadata.json";
    if(!$file || !is_file($file)) return false;
    $data = getJSONfromFile($file);

    // nach dem auslesen soll die datei entfernt werden
    rex_file::delete($file);

    return $data;
}

// bekomme json data / leeren array fals keine datei
function getJSONfromFile($file="") {
    if(!$file || !is_file($file)) return [];
    $json = file_get_contents($file);
    $json_data = json_decode($json,true);
    return $json_data;
}