<?php
// initialisiere addon

$addon = rex_addon::get("package_controller");


// import functions
require_once($addon->getPath() . "lib/scripts/download.php");
require_once($addon->getPath() . "lib/scripts/validation.php");
require_once($addon->getPath() . "lib/scripts/installation.php");
require_once($addon->getPath() . "lib/scripts/requirements.php");

// setzte $_POST
$rest_json = file_get_contents("php://input");
$_POST = json_decode($rest_json, true);


// debug
include_once($addon->getPath() . "lib/scripts/tests.php");

// lösche den ausgabepuffer
ob_clean();

// initialisiere respond array

$result = [
	"error" => false,
	"error_code" => 0,
	"data" => []
];

// funktion zum antworten auf die anfrage, parameter result behinhaltet array mit allen daten

function respond($result=[]) {
	header("Content-Type: application/json");
	echo json_encode($result);
	die();
}



function request($request = []) {
	$type = $request["type"];


	// init pathes
	$addon_root = rex_addon::get("package_controller")->getPath();
	$addon_data_root = rex_path::addonData("package_controller");
	$addon_data_modules = $addon_data_root . "modules/";
	$addon_data_templates = $addon_data_root . "templates/";
	$addon_data_tmp = $addon_data_root . "tmp/";

	// prüfe den type der anfrage

	if(!isset($type)) {
		throw new Exception("An API request was sent without having a type. This is only an internal error.", 1);
	}


	// type: downloadPackage

	if($type == "downloadPackage") {
		if(!isset($request["url"])) {
			throw new Exception("An API request was sent with a type of downloadPublic, but id did not sent an url. This is only an internal error.", 6);
		}

		// download
		if($tmp_folder = downloadPackage($request["url"], $addon_data_tmp)) {
			// validation
			validateTmpFolder($tmp_folder, $addon_data_root); 
			return;
		} 
			
		throw new Exception("An API request was sent with a type of downloadPublic, but the download failed. Wether the URL was incorrect or unzipping failed.", 7);
		
	}

	// type: delete

	if($type == "delete") {
		if(!isset($request["contentType"]) || !isset($request["id"])) {
			throw new Exception("An API request was sent with a type of delete, but the deletetion failed, because not all required parameters have been set.", 13);
		} 

		if($request["contentType"] == "package") {
			$package = new packageControllerPackage($request["id"]);
			$package->delete($addon_data_templates, $addon_data_modules);
			return;

		} else if($request["contentType"] == "template") {
			$temp = new packageControllerTemplate($request["id"]);
			$temp->delete($addon_data_templates);
			return;

		} else if($request["contentType"] == "module") {
			$mod = new packageControllerModule($request["id"]);
			$mod->delete($addon_data_modules);
			return;

		} 
			
		throw new Exception("An API request was sent with a type of delete, but the deletetion failed, because no valid contentType was given.", 14);
		
	}

	// type: getList

	if($type == "getList") {
		if(!isset($request["contentType"])) {
			throw new Exception("An API request was sent with a type of getList, but failed because not all parameters have been set.", 15);
		}

		if($request["contentType"] == "package") {
			return packageControllerPackage::getList();

		} else if($request["contentType"] == "template") {
			return packageControllerTemplate::getList();

		} else if($request["contentType"] == "module") {
			return packageControllerModule::getList();
			
		} else if($request["contentType"] == "public_package") {
			return packageControllerPublicPackage::getList();

		}
			
		throw new Exception("An API request was sent with a type of getList, but failed because a parameter had a wrong value.", 16);
		
	}

	// type: install

	if($type == "install") {
		if(!isset($request["contentType"]) || !isset($request["id"])) {
			throw new Exception("An API request was sent with a type of install, but failed because not all parameters have been set.", 17);
		}

		$username = (rex::getUser())->getName();

		if($request["contentType"] == "template") {
			installFolder($request["id"], $addon_data_templates, $username, "template");
			return;
		} else if($request["contentType"] == "module") {
			installFolder($request["id"], $addon_data_modules, $username, "module");
			return;
		}
			
		throw new Exception("An API request was sent with a type of install, but failed because a parameter had a wrong value.", 18);
	
	}

	// type getFileContent

	if($type == "getFileContent") {
		if(!isset($request["path"])) throw new Exception("An API request was sent with a type of getFileContent, but failed because not all parameter have been set.");
		
		$file = $addon_root . "/" . $request["path"];

		if(!is_file($file)) throw new Exception("There is no file with path: " . $file);

		if(!$fileContent = file_get_contents($file)) throw new Exception("There is no file with path: " . $file);
		
		return $fileContent;
	}


	// sollte bis jetzt noch keine passende antwort gefunden sein, so muss trotzdem eine antwort erfolgen

	throw new Exception("An API request was sent with a type that has no response. This is only an internal error.", 2);

}


// wrapper um request um Exceptions abzufangen
// exceptions werden in das ergbnis mit eingebaut
try {

	$result["data"] = request($_POST);

} catch(Exception $e) {
	
	$result["error"] = $e->getMessage();
	$result["error_code"] = $e->getCode();

} finally {
	respond($result);
}